@@ -23,8 +23,8 @@
  */
 
 #include	"mdadm.h"
-#include	"md_p.h"
-#include	"md_u.h"
+#include	"xmalloc.h"
+
 #include	<ctype.h>
 #include	<dirent.h>
 
@@ -49,6 +49,30 @@ static int add_device(const char *dev, char ***p_devices,
 	return n_devices + 1;
 }
 
+/**
+ * detail_fname_from_uuid() - generate uuid string with special super1 handling.
+ * @mp: map entry to parse.
+ * @buf: buf to write.
+ *
+ * Hack to workaround an issue with super1 superblocks. It swapuuid set in order for assembly
+ * to work, but can't have it set if we want this printout to match all the other uuid printouts
+ * in super1.c, so we force swapuuid to 1 to make our printout match the rest of super1.
+ *
+ * Always convert uuid if host is big endian.
+ */
+char *detail_fname_from_uuid(struct map_ent *mp, char *buf)
+{
+#if __BYTE_ORDER == BIG_ENDIAN
+	bool swap = true;
+#else
+	bool swap = false;
+#endif
+	if (strncmp(mp->metadata, "1.", 2) == 0)
+		swap = true;
+
+	return __fname_from_uuid(mp->uuid, swap, buf, ':');
+}
+
 int Detail(char *dev, struct context *c)
 {
 	/*
@@ -226,6 +250,9 @@ int Detail(char *dev, struct context *c)
 		str = map_num(pers, array.level);
 
 	if (c->export) {
+		char nbuf[64];
+		struct map_ent *mp = NULL, *map = NULL;
+
 		if (array.raid_disks) {
 			if (str)
 				printf("MD_LEVEL=%s\n", str);
@@ -247,38 +274,25 @@ int Detail(char *dev, struct context *c)
 				       array.minor_version);
 		}
 
-		if (st && st->sb && info) {
-			char nbuf[64];
-			struct map_ent *mp, *map = NULL;
-
-			fname_from_uuid(st, info, nbuf, ':');
-			printf("MD_UUID=%s\n", nbuf + 5);
+		if (info && memcmp(info->uuid, uuid_zero, sizeof(int[4])) != 0)
 			mp = map_by_uuid(&map, info->uuid);
-			if (mp && mp->path &&
-			    strncmp(mp->path, "/dev/md/", 8) == 0) {
-				printf("MD_DEVNAME=");
-				print_escape(mp->path + 8);
-				putchar('\n');
-			}
+		if (!mp)
+			mp = map_by_devnm(&map, fd2devnm(fd));
 
+		if (mp) {
+			detail_fname_from_uuid(mp, nbuf);
+			printf("MD_UUID=%s\n", nbuf + 5);
+			if (mp->path && strncmp(mp->path, DEV_MD_DIR, DEV_MD_DIR_LEN) == 0)
+				printf("MD_DEVNAME=%s\n", mp->path + DEV_MD_DIR_LEN);
+		}
+
+		map_free(map);
+		if (st && st->sb) {
+			if (info)
+				printf("MD_RESHAPE_ACTIVE=%s\n",
+				       info->reshape_active ? "True" : "False");
 			if (st->ss->export_detail_super)
 				st->ss->export_detail_super(st);
-			map_free(map);
-		} else {
-			struct map_ent *mp, *map = NULL;
-			char nbuf[64];
-			mp = map_by_devnm(&map, fd2devnm(fd));
-			if (mp) {
-				__fname_from_uuid(mp->uuid, 0, nbuf, ':');
-				printf("MD_UUID=%s\n", nbuf+5);
-			}
-			if (mp && mp->path &&
-			    strncmp(mp->path, "/dev/md/", 8) == 0) {
-				printf("MD_DEVNAME=");
-				print_escape(mp->path+8);
-				putchar('\n');
-			}
-			map_free(map);
 		}
 		if (!c->no_devices && sra) {
 			struct mdinfo *mdi;
@@ -303,6 +317,7 @@ int Detail(char *dev, struct context *c)
 				if (path)
 					printf("MD_DEVICE_%s_DEV=%s\n",
 					       sysdev, path);
+				free(sysdev);
 			}
 		}
 		goto out;
@@ -495,8 +510,8 @@ int Detail(char *dev, struct context *c)
 			if (array.state & (1 << MD_SB_CLEAN)) {
 				if ((array.level == 0) ||
 				    (array.level == LEVEL_LINEAR))
-					arrayst = map_num(sysfs_array_states,
-							  sra->array_state);
+					arrayst = map_num_s(sysfs_array_states,
+							       sra->array_state);
 				else
 					arrayst = "clean";
 			} else {
@@ -537,16 +552,10 @@ int Detail(char *dev, struct context *c)
 		} else if (inactive && !is_container) {
 			printf("             State : inactive\n");
 		}
-		if (array.raid_disks)
-			printf("    Active Devices : %d\n", array.active_disks);
-		if (array.working_disks > 0)
-			printf("   Working Devices : %d\n",
-			       array.working_disks);
-		if (array.raid_disks) {
-			printf("    Failed Devices : %d\n", array.failed_disks);
-			if (!external)
-				printf("     Spare Devices : %d\n", array.spare_disks);
-		}
+		printf("    Active Devices : %d\n", array.active_disks);
+		printf("   Working Devices : %d\n", array.working_disks);
+		printf("    Failed Devices : %d\n", array.failed_disks);
+		printf("     Spare Devices : %d\n", array.spare_disks);
 		printf("\n");
 		if (array.level == 5) {
 			str = map_num(r5layout, array.layout);
@@ -608,13 +617,6 @@ int Detail(char *dev, struct context *c)
 		}
 
 		if ((st && st->sb) && (info && info->reshape_active)) {
-#if 0
-This is pretty boring
-			printf("     Reshape pos'n : %llu%s\n",
-			       (unsigned long long) info->reshape_progress << 9,
-			       human_size((unsigned long long)
-					  info->reshape_progress << 9));
-#endif
 			if (info->delta_disks != 0)
 				printf("     Delta Devices : %d, (%d->%d)\n",
 				       info->delta_disks,
